/******************************************************************************
*
* project name:    TI-Memory
* file name:       board.c
* initial date:    13/11/2000
* authors:         thomas.nussbaumer@gmx.net
*                  marcos.lopez@wol.es
*
* description:     contains board handling and drawing functions
*
* $Id: board.c,v 1.2 2002/11/14 14:02:53 nussbaumer Exp $
*
******************************************************************************/

#define BOARD_SIZE_X    10   // don't change this
#define BOARD_SIZE_Y     6   // don't change this
#define SHUFFLE_ROUNDS 500   // number of random exchangements
#define NR_PIECES       32   // change this if you have more pieces

#define BOARD_FIELDS (BOARD_SIZE_X*BOARD_SIZE_Y)

void InitCursorBuffer();
void PrintTime();


// offsets (mainly used for TI92plus)
short offsetx = 0;
short offsety = 0;

// the board and the back buffers
short*           board         = NULL;
unsigned char* backbuffer[2] = {NULL,NULL};

// buffers for cursor background backup
unsigned char* cursor_buf[2] = {NULL,NULL};

// how much pairs are left
unsigned short pairs_left = 0;

// variables to store selected stones
short sel_x  = -1;
short sel_y  = -1;
short sel2_x = -1;
short sel2_y = -1;


short used_setup = 1;
short cur_min_x  = 0;
short cur_min_y  = 0;
short cur_max_x  = BOARD_SIZE_X;
short cur_max_y  = BOARD_SIZE_Y;


/*---------------------------------------------------------------------------*/
/* here comes the sprite data ...                                            */
/*---------------------------------------------------------------------------*/
#include "tiles.h"

unsigned short mask[16] = {
    0xc000,0x8000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0001,0x0003
};


unsigned short mask_out[16] = {
    0xFFFF,0xE001,0xE001,0xE001,0xE001,0xE001,0xE001,0xE001,
    0xE001,0xE001,0xE001,0xE001,0xE001,0xFFFF,0xFFFF,0xFFFF
};


/*===========================================================================*/
/* allocate space for some necessary globals                                 */
/*===========================================================================*/
short AllocateVariables() {
    if (!(board = malloc(BOARD_FIELDS*sizeof(short)))) return 0;
    if (!(backbuffer[0] = malloc(LCD_SIZE)))         return 0;
    if (!(backbuffer[1] = malloc(LCD_SIZE)))         return 0;
    if (!(cursor_buf[0] = malloc(250)))              return 0;
    if (!(cursor_buf[1] = malloc(250)))              return 0;

    if (TI89) offsetx = offsety = 0;
    else      offsetx = (240-160)/2,offsety = (128-100)/2;

    return 1;
}


/*===========================================================================*/
/* free allcated global again (CALL ONLY ONCE !!!)                           */
/*===========================================================================*/
void FreeVariables() {
    if (board)         free(board);
    if (backbuffer[0]) free(backbuffer[0]);
    if (backbuffer[1]) free(backbuffer[1]);
    if (cursor_buf[0]) free(cursor_buf[0]);
    if (cursor_buf[1]) free(cursor_buf[1]);
}



/*===========================================================================*/
/* initialize the board                                                      */
/*===========================================================================*/
void InitBoard() {
    short  i,x,y;
    short *tmpboard  = board;
    short  count     = BOARD_FIELDS;
    short  pieces[BOARD_FIELDS];

    switch(used_setup) {
        case 3:
            count      = BOARD_FIELDS;
            cur_min_x  = 0;
            cur_min_y  = 0;
            cur_max_x  = BOARD_SIZE_X;
            cur_max_y  = BOARD_SIZE_Y;
            break;
        case 2:
            cur_min_x  = 1;
            cur_min_y  = 1;
            cur_max_x  = BOARD_SIZE_X-1;
            cur_max_y  = BOARD_SIZE_Y-1;
            count      = BOARD_FIELDS - 28;
            break;
        case 1:
            cur_min_x  = 2;
            cur_min_y  = 1;
            cur_max_x  = BOARD_SIZE_X-2;
            cur_max_y  = BOARD_SIZE_Y-1;
            count      = BOARD_FIELDS - 28 - 8;
            break;
        case 0:
            cur_min_x  = 3;
            cur_min_y  = 1;
            cur_max_x  = BOARD_SIZE_X-3;
            cur_max_y  = BOARD_SIZE_Y-1;
            count      = BOARD_FIELDS - 28 - 16;
            break;
    }

    sel_x = sel_y = -1;

    pairs_left = count / 2;

    //----------------------------------------------------
    // fill pieces array with piece pairs
    //----------------------------------------------------
    for (i=0;i<count/2;i++) {
        pieces[i*2]   = i % NR_PIECES;
        pieces[i*2+1] = i % NR_PIECES;
    }

    //----------------------------------------------------
    // exchange random pieces with each other
    //----------------------------------------------------
    for (i=0;i<SHUFFLE_ROUNDS;i++) {
        short select1 = random(count);
        short select2 = random(count);
        short tmp     = pieces[select1];

        pieces[select1] = pieces[select2];
        pieces[select2] = tmp;
    }

    //----------------------------------------------------
    // fill up the board
    //----------------------------------------------------
    for (i=0;i<BOARD_FIELDS;i++) *tmpboard++ = -1;

    for (y=cur_min_y;y<cur_max_y;y++) {
        for (x=cur_min_x;x<cur_max_x;x++) {
            *(board+y*BOARD_SIZE_X+x) = pieces[--count];
        }
    }
}


/*===========================================================================*/
/* draws a single field                                                      */
/*===========================================================================*/
void DrawField(short x,short y) {
    short i;

    short which = board[y*BOARD_SIZE_X+x];

    if (which != -1) {
        short offset = which * 16;
        short xx = offsetx+x*13+11;
        short yy = offsety+y*13+10;
        for (i=0;i<2;i++) {
            Sprite16(xx,yy,16,mask,             backbuffer[i],SPRT_AND);
            Sprite16(xx,yy,16,&tiles[i][offset],backbuffer[i],SPRT_OR);
            if ((x != sel_x || y != sel_y) && (x != sel2_x || y != sel2_y)) {
                Sprite16(xx,yy,16,mask_out,backbuffer[i],SPRT_AND);
            }
        }
    }
}


/*===========================================================================*/
/* generates board display in backbuffer and copies it to display afterwards */
/*===========================================================================*/
void DrawBoard() {
    short x,y;

    for (x=0;x<2;x++) memset(backbuffer[x],0,LCD_SIZE);

    for (y=0;y<BOARD_SIZE_Y;y++) {
        for (x=BOARD_SIZE_X-1;x>=0;x--) {
            DrawField(x,y);
        }
    }

    for (x=0;x<2;x++) memcpy(GetPlane(x),backbuffer[x],LCD_SIZE);
    InitCursorBuffer();
    PrintTime();
}


/*===========================================================================*/
/* useful macros to get and set stones                                       */
/*===========================================================================*/
#define GetStone(x,y) board[BOARD_SIZE_X*(y)+(x)]
#define SetStone(x,y,t) board[BOARD_SIZE_X*(y)+(x)] = (t)


/*===========================================================================*/
/* unselects selected stone                                                  */
/*===========================================================================*/
void Unselect() {
    sel_x = sel_y = sel2_x = sel2_y = -1;
    DrawBoard();
}


/*===========================================================================*/
/* check if game is won (if there are still stones on the board)             */
/*===========================================================================*/
short CheckWin() {
    short i;
    short *b = board;

    for (i=0;i<BOARD_FIELDS;i++) {
        if (*b++ != -1) return 0;
    }

    return 1;
}


/*===========================================================================*/
/* treatment of a select stone event                                         */
/*===========================================================================*/
short HandleSelect(short x,short y) {
    if (GetStone(x,y) == -1) return 0;

    //-----------------------------
    // new selected stone ...
    //-----------------------------
    if (sel_x == -1) {
        sel_x = x;
        sel_y = y;
        DrawBoard();
        return 0;
    }
    else if (sel_x != -1 && sel2_x != -1) {
        Unselect();
        sel_x = x;
        sel_y = y;
        DrawBoard();
        return 0;
    }
    //-----------------------------------------
    // check types and remove them if equal ...
    //-----------------------------------------
    else if (sel_x == x && sel_y == y) {
        return 0;
    }
    else {
        short type1 = GetStone(sel_x,sel_y);
        short type2 = GetStone(x,y);

        if (type1 == type2) {
            SetStone(sel_x,sel_y,-1);
            SetStone(x,y,-1);
            pairs_left--;
            Unselect();
        }
        else {
            sel2_x = x;
            sel2_y = y;
            DrawBoard();
            return 0;
        }
    }

    return CheckWin();
}


//=============================================================================
// Revision History
//=============================================================================
//
// $Log: board.c,v $
// Revision 1.2  2002/11/14 14:02:53  nussbaumer
// changes for AMS 2.08/V200 support and for TIGCC 0.94
//
// Revision 1.1  2000/11/14 19:34:38  Thomas Nussbaumer
// initial version
//
//
