/******************************************************************************
*
* project name:    TI-Memory
* file name:       memory.c
* initial date:    05/11/2000
* authors:         thomas.nussbaumer@gmx.net
*                  marcos.lopez@gmx.net
*
* description:     main program
*
* compile with: tigcc -O2 memory.c
*
*
* NOTE:  No animals or trees were harmed for the stones of this game
*
*
* $Id: memory.c,v 1.2 2002/11/14 14:02:53 nussbaumer Exp $
*
******************************************************************************/

#define SAVE_SCREEN
#define OPTIMIZE_ROM_CALLS
#define NO_EXIT_SUPPORT

#define USE_TI89
#define USE_TI92PLUS
#define USE_V200

#include <tigcclib.h>


#define MEMORY_VERSION "v1.10"

#include "board.c"
#include "cursor.c"
#include "clock.c"

/*---------------------------------------------------------------------------*/
/* some globals                                                              */
/*---------------------------------------------------------------------------*/
SCR_RECT fullscreen = {{0,0,239,127}}; // helper variable for clipping


/*---------------------------------------------------------------------------*/
/* some defines for drawing functions                                        */
/*---------------------------------------------------------------------------*/
#define COLOR_WHITE      0
#define COLOR_LIGHTGRAY  1
#define COLOR_DARKGRAY   2
#define COLOR_BLACK      3
#define LIGHT_PLANE      0
#define DARK_PLANE       1
#define RECT_EMPTY       0
#define RECT_FILLED      1
#define A_CENTERED       0x40
#define A_SHADOWED       0x80

short cursor_x = 0;
short cursor_y = 0;


/*===========================================================================*/
/* utility funcction to draw a string to both planes                         */
/* NOTE: there exists to new styles which are handled as flags               */
/*       A_CENTERED ... the given font will be used and the string will be   */
/*                      drawn centered on the screen                         */
/*       A_SHADOWED ... if this flag is set in the attributes a lightgray    */
/*                      shadow will be drawn with +1 pixel offset in all     */
/*                      two directions                                       */
/*===========================================================================*/
void DrawString(short x,short y,char* s,short font,short attr) {
    short do_shadow = attr & A_SHADOWED;

    x += offsetx;
    y += offsety;

    FontSetSys(font);

    if (attr & A_CENTERED) {
        x = (LCD_WIDTH-DrawStrWidth(s,font))/2;
        attr &= ~A_CENTERED;
    }

    attr &= ~A_SHADOWED;

    SetPlane(DARK_PLANE);
    DrawStr(x,y,s,attr);
    SetPlane(LIGHT_PLANE);
    DrawStr(x,y,s,attr);

    if (do_shadow) DrawStr(x+1,y+1,s,A_NORMAL);
}


/*===========================================================================*/
/* draw a line with given color from (x0,y0) to (x1,y1)                      */
/*===========================================================================*/
void DrawColorLine(short x0,short y0,short x1, short y1, short color) {
    SetPlane(LIGHT_PLANE);
    if (color == COLOR_WHITE || color == COLOR_DARKGRAY) {
        DrawLine(x0, y0, x1, y1, A_REVERSE);
    }
    else {
        DrawLine(x0, y0, x1, y1, A_NORMAL);
    }
    SetPlane(DARK_PLANE);
    if (color == COLOR_WHITE || color == COLOR_LIGHTGRAY) {
        DrawLine(x0, y0, x1, y1, A_REVERSE);
    }
    else {
        DrawLine(x0, y0, x1, y1, A_NORMAL);
    }
}


/*===========================================================================*/
/* draws a rectangle with given color                                        */
/* if fill == 1) the rectangle is filled                                     */
/*===========================================================================*/
void DrawColorRect(short x0,short y0,short x1, short y1, short color,short fill) {
    WIN_RECT wrect;
    SCR_RECT rect;
    void     (*df)(void*,void*,short);
    void*    parea;

    if (fill) {
        rect.xy.x0 = x0;
        rect.xy.y0 = y0;
        rect.xy.x1 = x1;
        rect.xy.y1 = y1;
        parea      = &rect;
        df         = (void(*)(void*,void*,short))ScrRectFill;
    }
    else {
        wrect.x0 = x0;
        wrect.y0 = y0;
        wrect.x1 = x1;
        wrect.y1 = y1;
        parea    = &wrect;
        df       = (void(*)(void*,void*,short))DrawClipRect;
    }

    SetPlane(LIGHT_PLANE);
    if (color == COLOR_WHITE || color == COLOR_DARKGRAY) {
        (*df)(parea,&fullscreen, A_REVERSE);
    }
    else {
        (*df)(parea,&fullscreen, A_NORMAL);
    }
    SetPlane(DARK_PLANE);
    if (color == COLOR_WHITE || color == COLOR_LIGHTGRAY) {
        (*df)(parea,&fullscreen, A_REVERSE);
    }
    else {
        (*df)(parea,&fullscreen, A_NORMAL);
    }
}


/*===========================================================================*/
/* draws a popup window                                                      */
/*===========================================================================*/
void DrawPopup(short x0,short y0,short x1, short y1) {
    DrawColorRect(x0,y0,x1,y1,COLOR_BLACK,RECT_EMPTY);
    x0++,y0++,x1--,y1--;
    DrawColorRect(x0,y0,x1,y1,COLOR_DARKGRAY,RECT_EMPTY);
    x0++,y0++,x1--,y1--;
    DrawColorRect(x0,y0,x1,y1,COLOR_LIGHTGRAY,RECT_EMPTY);
    x0++,y0++,x1--,y1--;
    DrawColorRect(x0,y0,x1,y1,COLOR_WHITE,RECT_FILLED);
}


/*===========================================================================*/
/* outputs the time to the statusline                                        */
/*===========================================================================*/
void PrintTime(void) {
    char buffer[16];
    sprintf(buffer,"%02d:%02d:%02d", hours, minutes, seconds);
    DrawString(130,95,buffer,F_4x6,A_REPLACE);
}


/*===========================================================================*/
/* get input key pressed by user                                             */
/*                                                                           */
/* replacement for ngetchx() and/or GKeyIn(), because:                       */
/*                                                                           */
/* (1) ngetchx() won't check the APD timer value                             */
/* (2) GKeyIn() destroys the display due to setting the status display       */
/*                                                                           */
/* (handles also time printing in its inner loop)                            */
/*===========================================================================*/
short GetUserInput() {
    void *kbq = kbd_queue();
    short key;

    //-------------------------------------------
    // restart the APD timer now
    //-------------------------------------------
    OSTimerRestart(APD_TIMER);

    while (1) {
        //---------------------------------------
        // wait until we get a key
        // if the APD timer expires -> turn
        // calculator off
        //---------------------------------------
        while (OSdequeue(&key, kbq)) {
            if (OSTimerExpired(APD_TIMER)) {
                off();
                OSTimerRestart(APD_TIMER);
            }
            if (outputtime) {
                outputtime = 0;
                PrintTime();
            }
        }

        //---------------------------------------
        // handle calculator-off key combinations
        // if the APD timer expires -> turn
        // calculator off
        //---------------------------------------
        if (key == 4363 || (TI89 && key == 16651) || (!TI89 && key == 8459)) {
            off();
            OSTimerRestart(APD_TIMER);
        }
        else {
            return key & 0xf7ff;  // mask out 0x800 (repeat flag)
        }
    }
}


#include "logo.c"

/*===========================================================================*/
/* treatment of cursor position changement                                   */
/*===========================================================================*/
void CursorUpdate(void) {
    if (cursor_x >= cur_max_x) cursor_x = cur_min_x;
    if (cursor_x < cur_min_x)  cursor_x = cur_max_x-1;
    if (cursor_y >= cur_max_y) cursor_y = cur_min_y;
    if (cursor_y < cur_min_y)  cursor_y = cur_max_y-1;

    RestoreOldCursorPos();
    BackupCursorPos(cursor_x,cursor_y);
    OutputCursor(cursor_x,cursor_y);

    {
        char buf[100];
        sprintf(buf,"[F5 for Help]      Pairs Left = %d",pairs_left);

        DrawString(0,95,buf,F_4x6,A_REPLACE);
    }
}


unsigned char* helppage[] = {
    "To win this Game You have to remove all",
    "Stones from the Board.",
    "Stones can only be removed in Pairs which",
    "must have the same Graphics on them.",
    "",
    "",
    "Use the Cursor Keys and [ENTER] or [F1] to",
    "select the Stones.",
    "",
    "[CLEAR] to Reset   [+][-] to change Setup",
    NULL};

/*===========================================================================*/
/* displays help page                                                        */
/*===========================================================================*/
void DisplayHelp() {
    short i = 0;

    running    = 0;
    outputtime = 0;

    memset(GetPlane(0),0,LCD_SIZE);
    memset(GetPlane(1),0,LCD_SIZE);


    DrawString(0,0,"TI-Memory "MEMORY_VERSION" by TICT",F_6x8,A_REPLACE | A_SHADOWED);

    while (helppage[i]) {
        DrawString(0,20+i*7,helppage[i],F_4x6,A_REPLACE);
        i++;
    }
    GetUserInput();

    running    = 1;
    outputtime = 1;
}


/*===========================================================================*/
/* initialize a new game (returns 1 if okay)                                 */
/*===========================================================================*/
void InitGame(void) {
    randomize();

    resettime = 1;
    running   = 1;

    InitBoard();
    cursor_x  = cur_min_x;
    cursor_y  = cur_max_y;

    DrawBoard();
    PrintTime();
    CursorUpdate();
}


/*===========================================================================*/
/* draws a message                                                           */
/*===========================================================================*/
void DrawMessage(char* msg) {
    short sy = LCD_HEIGHT/2 - 2 + 44;

    DrawPopup(1,sy-6,LCD_WIDTH-1,sy+7);
    DrawString(0,sy-3-offsety,msg,F_6x8,A_REPLACE | A_CENTERED | A_SHADOWED);

    GetUserInput();
}



/*===========================================================================*/
/* as usual our main routine ...                                             */
/*===========================================================================*/
void _main(void) {
    short key;

    if (!GrayOn()) return;

    if (!AllocateVariables()) {
        GrayOff();
        return;
    }

    outputtime = mseconds50 = seconds = minutes = hours = 0;

    Logo();

    InitGame();

    oldint5 = GetIntVec(AUTO_INT_5);
    SetIntVec(AUTO_INT_5, myint5handler);

    do {
        key = GetUserInput();

        if (sel_x != -1 && sel2_x != -1) {
            Unselect();
            DrawBoard();
        }

        if (key == KEY_UP) {
            cursor_y--;
            CursorUpdate();
        }
        else if (key == KEY_DOWN) {
            cursor_y++;
            CursorUpdate();
        }
        else if (key == KEY_LEFT) {
            cursor_x--;
            CursorUpdate();
        }
        else if (key == KEY_RIGHT) {
            cursor_x++;
            CursorUpdate();
        }
        else if (key == KEY_ENTER || key == KEY_F1) {
            if (HandleSelect(cursor_x,cursor_y)) {
                //---------------------------
                // !!! THE PLAYER HAS WON !!!
                //---------------------------
                char msg[100];
                running    = 0;
                outputtime = 0;
                sprintf(msg,"You've won in %02d:%02d:%02d !", hours, minutes, seconds);
                DrawMessage(msg);
                InitGame();
            }
            else {
                CursorUpdate();
            }
        }
        else if (key == KEY_CLEAR || key == '+' || key == '-') {
            if (key == '+') used_setup++;
            if (key == '-') used_setup--;

            if (used_setup < 0) used_setup = 3;
            if (used_setup > 3) used_setup = 0;

            InitGame();
        }
        else if (key == KEY_F5) {
            DisplayHelp();
            DrawBoard();
            CursorUpdate();
        }
    }
    while (key != KEY_ESC);

    FreeVariables();
    SetIntVec(AUTO_INT_5, oldint5);
    GrayOff();
}


//=============================================================================
// Revision History
//=============================================================================
//
// $Log: memory.c,v $
// Revision 1.2  2002/11/14 14:02:53  nussbaumer
// changes for AMS 2.08/V200 support and for TIGCC 0.94
//
// Revision 1.1  2000/11/14 19:34:38  Thomas Nussbaumer
// initial version
//
//
